import React from 'react';
import { createStyledComponent } from '@cloudflare/style-container';
import { RowProps } from './types';
import Cell from './Cell';

type RowWrapperProps = {
  index: number;
  rowHighlight?: boolean;
  striped?: boolean;
  expandable: boolean;
  disabled?: boolean;
  isSelected?: boolean;
  isLoading?: boolean;
  rowStyle?: object;
  testId?: string;
};

const RowWrapper = createStyledComponent<'div', RowWrapperProps>(
  ({
    index,
    theme,
    rowHighlight,
    striped,
    expandable,
    disabled,
    isSelected,
    isLoading,
    rowStyle
  }) => ({
    position: 'relative',
    backgroundColor: theme.colors.background,
    ...(index > 0 && { borderTop: `1px solid ${theme.colors.gray[7]}` }),
    ...(isSelected && { backgroundColor: 'blue.9' }),
    ...(!isLoading && disabled && { backgroundColor: 'gray.9' }),
    ...(expandable && { cursor: 'pointer' }),
    ...(striped && index % 2 !== 0
      ? {
          backgroundColor: 'gray.8'
        }
      : {}),
    ...(rowHighlight
      ? {
          transition: 'background-color 0.07s ease-out',
          '&:hover': { backgroundColor: 'gray.7' }
        }
      : {}),
    ...(rowStyle || {})
  })
);

export const DefaultExpandable = createStyledComponent(({ theme }) => ({
  position: 'sticky',
  left: 0,
  px: 1,
  '&:before': {
    content: '""',
    position: 'absolute',
    left: 0,
    top: 0,
    height: '100%',
    width: '4px',
    backgroundColor: theme.colors.blue[3],
    zIndex: 1
  }
}));

const Row = <Data extends object = {}>({
  row,
  rowId,
  rowStyle,
  isLoading,
  expandable,
  isRowExpandable,
  isRowSelectable,
  onlyOneExpanding,
  toggleAllRowsExpanded,
  striped,
  rowHighlight,
  rowIndex,
  onRowClick,
  containerWidth,
  disabled,
  testId
}: RowProps<Data>) => {
  const isExpandable =
    typeof isRowExpandable === 'function' && !!expandable
      ? isRowExpandable({ row })
      : !!expandable;

  const onClickFn = () => {
    onRowClick({
      isExpanding: !row.isExpanded,
      row
    });
    if (onlyOneExpanding && !row.isExpanded) {
      toggleAllRowsExpanded(false);
      row.toggleRowExpanded(true);
    } else {
      row.toggleRowExpanded();
    }
  };

  const ariaProps = isExpandable
    ? {
        tabIndex: 0,
        'aria-expanded': !!row.isExpanded,
        onClick: onClickFn,
        onKeyDown: (event: React.KeyboardEvent<HTMLDivElement>) => {
          if (event.key === 'Enter' || event.key === ' ') {
            event.preventDefault();
            onClickFn();
          }
        }
      }
    : {};

  return (
    <RowWrapper
      data-testid={testId}
      index={rowIndex}
      rowStyle={rowStyle}
      rowHighlight={rowHighlight}
      striped={striped}
      expandable={isExpandable}
      disabled={disabled}
      isSelected={row.isSelected}
      isLoading={isLoading}
    >
      <div {...row.getRowProps()} {...ariaProps}>
        {row.cells.map((cell, index) => (
          <Cell
            key={cell.getCellProps()?.key}
            id={`${rowId}_cell_${index}`}
            isLoading={isLoading}
            // @ts-ignore
            cell={cell}
            isExpandable={isExpandable}
            isRowSelectable={isRowSelectable}
          />
        ))}
      </div>

      {row.isExpanded &&
        !isLoading &&
        // @ts-ignore
        expandable?.({ row, maxWidth: containerWidth })}
    </RowWrapper>
  );
};

export default Row;
